@@ -32,11 +32,11 @@ Test::Aggregate - Aggregate C<*.t> tests to make them run faster.
 
 =head1 VERSION
 
-Version 0.34_03
+Version 0.34_04
 
 =cut
 
-$VERSION = '0.34_03';
+$VERSION = '0.34_04';
 
 =head1 SYNOPSIS
 
@@ -182,6 +182,12 @@ before each test file.
 
 This is turned off by default.
 
+=item * C<dry> (optional)
+
+Just print the tests which will be run and the order they will be run in
+(obviously the order will be random if C<shuffle> is true).
+
+
 =item * C<tidy>
 
 If supplied a true value, attempts to run C<Perl::Tidy> on the source code.
@@ -269,6 +275,7 @@ sub new {
     $self->{$_} = delete $arg_for->{$_} foreach (
         qw/
         dump
+        dry
         set_filenames
         findbin
         shuffle
@@ -306,6 +313,7 @@ we cannot load Data::Dump::Streamer:  $error.
 
 sub _check_plan      { shift->{check_plan} || 0 }
 sub _dump            { shift->{dump} || '' }
+sub _dry             { shift->{dry} }
 sub _should_shuffle  { shift->{shuffle} }
 sub _matching        { shift->{matching} }
 sub _set_filenames   { shift->{set_filenames} }
@@ -362,7 +370,17 @@ sub _shuffle {
 sub run {
     my $self  = shift;
 
-    my $code = $self->_build_aggregate_code;
+    my @tests = $self->_get_tests;
+    if ( $self->_dry ) {
+        my $current = 1;
+        my $total   = @tests;
+        foreach my $test (@tests) {
+            print "$test (File $current out of $total)\n";
+            $current++;
+        }
+        return;
+    }
+    my $code = $self->_build_aggregate_code(@tests);
 
     my $dump = $self->_dump;
     if ( $dump ne '' ) {
@@ -371,7 +389,15 @@ sub run {
         print FH $code;
         close FH;
     }
+
+    # XXX Theoretically the 'eval $code' could run the tests directly and
+    # remove a lot of annoying duplication, but unfortunately, we can't
+    # properly capture the startup/shutdown/setup/teardown behavior there
+    # without mandating that Data::Dump::Streamer be installed.  As a result,
+    # this eval'ed code has a check to not actually run the tests if we are
+    # not in the dump file.
     eval $code;
+
     if ( my $error = $@ ) {
         croak("Could not run tests: $@");
     }
@@ -398,21 +424,63 @@ sub run {
 }
 
 sub _build_aggregate_code {
-    my $self = shift;
+    my ( $self, @tests ) = @_;
     my $code = $self->_test_builder_override;
 
     my ( $startup,  $startup_code )  = $self->_as_code('startup');
     my ( $shutdown, $shutdown_code ) = $self->_as_code('shutdown');
     my ( $setup,    $setup_code )    = $self->_as_code('setup');
     my ( $teardown, $teardown_code ) = $self->_as_code('teardown');
+    
     my $verbose = $self->_verbose;
-
+    my $findbin;
+    if ( $self->_findbin ) {
+        $findbin = <<'        END_CODE';
+use FindBin;
+my $REINIT_FINDBIN = FindBin->can(q/again/) || sub {};
+        END_CODE
+    }
+    else {
+        $findbin = 'my $REINIT_FINDBIN = sub {};';
+    }
     $code .= <<"    END_CODE";
 $startup_code
 $shutdown_code
 $setup_code
 $teardown_code
 my \$LAST_TEST_NUM = 0;
+$findbin
+
+sub ::see_if_tests_passed {
+    my ( \$test_name, \$verbose ) = \@_;
+    my \$tests   = \$BUILDER->current_test;
+    my \$failed = 0;
+    my \@summary = \$BUILDER->summary;
+    foreach my \$passed ( \@summary[\$LAST_TEST_NUM .. \$tests - 1] ) {
+        if ( not \$passed ) {
+            \$failed = 1;
+            last;
+        }
+    }
+    my \$ok = \$failed ? "not ok - \$test_name" : "    ok - \$test_name";
+    if ( \$failed or \$verbose == $VERBOSE{all} ) {
+        Test::More::diag(\$ok);
+    }
+    \$LAST_TEST_NUM = \$tests;
+}
+
+sub ::run_this_test_program {
+    my ( \$package, \$test ) = \@_;
+    Test::More::diag("******** running tests for \$test ********") if \$ENV{TEST_VERBOSE};
+    eval { \$package->run_the_tests };
+
+    return unless my \$error = \$@;
+    Test::More::ok( 0, "Error running (\$test):  \$error" );
+    # XXX this should be fine since these keys are not actually used
+    # internally.
+    \$BUILDER->{XXX_test_failed}       = 0;
+    \$BUILDER->{TEST_MOST_test_failed} = 0;
+}
     END_CODE
     
     my @packages;
@@ -425,13 +493,16 @@ my \$LAST_TEST_NUM = 0;
     $code .= <<"    END_CODE";
 if ( __FILE__ eq '$dump' ) {
     package Test::Aggregate; # ;)
-    my \$builder = Test::Builder->new;
     END_CODE
 
     if ( $startup ) {
         $code .= "    $startup->() if __FILE__ eq '$dump';\n";
     }
-    foreach my $test ($self->_get_tests) {
+
+    my $current_test = 0;
+    my $total_tests  = @tests;
+    foreach my $test (@tests) {
+        $current_test++;
         my $test_code = $self->_slurp($test);
 
         # get rid of hashbangs as Perl::Tidy gets all huffy-like and we
@@ -450,17 +521,8 @@ if ( __FILE__ eq '$dump' ) {
         if ( $setup ) {
             $code .= "    $setup->('$test');\n";
         }
-        $code .= <<"        END_CODE";
-    Test::More::diag("******** running tests for $test ********") if \$ENV{TEST_VERBOSE};
-    eval { $package->run_the_tests };
-    if ( my \$error = \$@ ) {
-        Test::More::ok( 0, "Error running ($test):  \$error" );
-        # XXX this should be fine since these keys are not actually used
-        # internally.
-        \$builder->{XXX_test_failed}       = 0;
-        \$builder->{TEST_MOST_test_failed} = 0;
-    }
-        END_CODE
+        $code .= qq{    ::run_this_test_program( $package => "$test" );};
+
         if ( $teardown ) {
             $code .= "    $teardown->('$test');\n";
         }
@@ -469,47 +531,28 @@ if ( __FILE__ eq '$dump' ) {
         my $set_filenames = $self->_set_filenames
             ? "local \$0 = '$test';"
             : '';
-        my $findbin = $self->_findbin ? <<'        END_CODE' : '';
-my $reinit_findbin = FindBin->can(q/again/);
-$reinit_findbin->() if $reinit_findbin;
-        END_CODE
-        my $see_if_tests_passed = $verbose ? <<"        END_CODE" : '';
-{
-    my \$builder = Test::Builder->new;   # singleton
-    my \$tests   = \$builder->current_test;
-    my \$failed = 0;
-    my \@summary = \$builder->summary;
-    foreach my \$passed ( \@summary[\$LAST_TEST_NUM .. \$tests - 1] ) {
-        if ( not \$passed ) {
-            \$failed = 1;
-            last;
-        }
-    }
-    my \$ok = \$failed ? "not ok - $test" : "    ok - $test";
-    if ( \$failed or $verbose == $VERBOSE{all} ) {
-        Test::More::diag(\$ok);
-    }
-    \$LAST_TEST_NUM = \$tests;
-}
-        END_CODE
+
+        my $test_name = "$test ($current_test out of $total_tests)";
+        my $see_if_tests_passed = $verbose 
+            ? qq{        ::see_if_tests_passed( "$test_name", $verbose );}
+            : '';
         $test_packages .= <<"        END_CODE";
 {
 $separator beginning of $test $separator
     package $package;
     sub run_the_tests {
         AGGTESTBLOCK: {
-        if ( my \$reason = \$Test::Aggregate::Builder::SKIP_REASON_FOR{$package} )
-        {
+        if ( my \$reason = \$Test::Aggregate::Builder::SKIP_REASON_FOR{$package} ) {
             Test::Builder->new->skip(\$reason);
             last AGGTESTBLOCK;
         }
-\$Test::Aggregate::Builder::FILE_FOR{$package} = '$test';
-$set_filenames
-$findbin
+        \$Test::Aggregate::Builder::FILE_FOR{$package} = '$test';
+        $set_filenames
+        \$REINIT_FINDBIN->();
 # line 1 "$test"
 $test_code
-$see_if_tests_passed
         } # END AGGTESTBLOCK:
+$see_if_tests_passed
     }
 $separator end of $test $separator
 }
@@ -599,6 +642,8 @@ BEGIN {
 use Test::Aggregate::Builder;
 BEGIN { \$Test::Aggregate::Builder::CHECK_PLAN = $check_plan };
 $disable_test_nowarnings;
+
+my \$BUILDER = Test::Builder->new;
     END_CODE
 }
 
